*=======================================================================*
*                                                                       *
*                       S T E P   D I S W A G                           *
*                                                                       *
*    Distance Wagner procedure (Farris, 1972) and modified distance     *
*                Wagner procedure (Swofford, 1981)                      *
*                                                                       *
*  Note: Nei distances are not allowed as input, since they do not      *
*  satisfy the triangle inequality and thus create problems for the     *
*  algorithm.  Only those distance measures which are metric (Rogers,   *
*  Prevosti,  Cavalli-Sforza and Edwards, and Edwards) are permitted.   *
*                                                                       *
*  Optimization of branch lengths follows Swofford (1981).  Other       *
*  methods for branch length optimization exist (see Swofford, in       *
*  prep.), but are not feasible for a general program.  Step SIMDIS has *
*  an option for creating a file that can be easily modified for input  *
*  to my "NETOPT" program, which performs branch length optimization    *
*  by least-squares and/or linear programming methods.                  *
*                                                                       *
*  Options and parameters:                                              *
*     NCOEF  ("COEF")    = coefficient to be used (see routine SIMDIS)  *
*     NADD   ("ADDCRIT") = 0  multiple addition criterion procedure of  *
*                                Swofford (1981)                        *
*                        = 1  use criterion 1 to determine addition     *
*                                sequence; use criterion 3 to break     *
*                                ties                                   *
*                        = 2  use criterion 2 to determine addition     *
*                                sequence; use criterion 3 to break     *
*                                ties                                   *
*                        = 3  use criterion 3 to determine addition     *
*                                sequence (default); (see Swofford,     *
*                                1981 for definition of addition        *
*                                criteria)                              *
*                                                                       *
*     NFIT   ("FITCRIT") = 1  minimize Prager and Wilson's F-value      *
*                                when selecting partial networks to     *
*                                be held for successive steps (default) *
*                        = 2  minimize percent standard deviation       *
*                        = 3  maximize cophenetic correlation           *
*     MAXTR  ("MAXTREE") = n  number of possible networks (out of 3n    *
*                                possible partial networks generated    *
*                                during each step) to be held and       *
*                                operated on in next step (default is   *
*                                n = 5)                                 *
*     NONEG              : do not permit negative branch lengths        *
*                                (i.e., set to 0)                       *
*     NROOT  ("ROOT")    = 1  root tree at midpoint of greatest         *
*                                patristic distance (default)           *
*                        = 2  root tree using specified outgroup        *
*                                (outgroup taxa must be identified on   *
*                                population header cards)               *
*     NOOPT  ("NOOPT")   : suppress branch length optimization          *
*     MATOUT  "PHENOUT") : print phenetic distance matrix               *
*     PATOUT ("PATOUT")  : print patristic distance matrix              *
*     NPLASY ("PLASOUT") : print homoplasy matrix                       *
*     STEPS              : print steps in computing tree                *
*     PLOT               : plot Wagner tree on Calcomp (or Zeta)        *
*                             plotter                                   *
*     YLIM               = maximum value for y-axis for tree drawn on   *
*                             Calcomp plotter (automatic scaling is     *
*                             the default; explicit setting of YLIM     *
*                             overrides automatic scaling)              *
*                                                                       *
*                                                                       *
*  Subprograms required by step DISWAG:                                 *
*                                                                       *
*     DISWAG  READER  BRANCH  TOPOL   TMPTRE  DCALC   SWITCH  CHANGE    *
*     CONGEN  NETWRK  TREE    DISOPT  STAND   ASUB    FIT     BCALC     *
*     PDCALC  MIDPNT  OUTGRP  TRPRIN  TRPLOT  PDROOT  DB      GFTEST    *
*     COMMAN  MDEX    MATPRT                                            *
*                                                                       *
*=======================================================================*

      SUBROUTINE DISWAG
     +   (D,P,DPHEN,HPLASY,POPNAM,ANC,NTERM,TERMB,NCONN,NWT,
     +    IGRP,ITAX,CD,CUSE,IVECT,ICLINE,ICBEG,ICEND,
     +    NPOS,NCENT,NLINE,NORDER,LINKED,
     +    MT,MT2,MHLFMT,ANCTOP,ANCPRV,DWRECL)

*  Performs distance Wagner procedure (Farris,1972) and modified
*  procedure using multiple addition criterion ("MAC") approach
*  (Swofford, 1981)

      IMPLICIT INTEGER (B,X-Z)
      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,
     +   INTST*64,REALST*8,LOGST*8,COEF*48,POPNAM*16
      REAL D,P,DPHEN,HPLASY,TERMB,CD,CUSE,FITNS,M,MINDB,MI,REALVL
      INTEGER NWT,ANC,ANCTOP,ANCPRV,NTERM,NCONN,IGRP,ITAX,IVECT,ICLINE,
     +   ICBEG,ICEND,NPOS,NCENT,NLINE,NORDER,LINKED,
     +   ZADD,HYPANC,OLDNOD,DWRECL
      LOGICAL SICRT,NONEG,NOOPT,MATOUT,PATOUT,NPLASY,STEPS,PLOT,
     +   LOGVL,CRT
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION POPNAM(MT),INTST(5),LOGST(7),D(MHLFMT),P(MHLFMT),
     +   DPHEN(MT,MT),HPLASY(MT,MT),TERMB(MT),CD(MT),CUSE(MT),
     +   NWT(MT2),ANC(MT2),ANCTOP(MT2),ANCPRV(MT2),NTERM(MT2),
     +   NCONN(MT2,3),IGRP(MT),ITAX(MT,2),IVECT(101),ICLINE(MT,2),
     +   ICBEG(MT,2),ICEND(MT,2),NPOS(MT),NCENT(MT),NLINE(MT),
     +   NORDER(MT),LINKED(MT),ZADD(3),FITNS(30),REALST(1),INTVL(5),
     +   LOGVL(7),REALVL(7)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP

      DATA INTST/'COEF','ADDCRIT','FITCRIT','MAXTREE','ROOT'/,
     +     REALST/'YLIM'/,
     +     LOGST/'NONEG','NOOPT','PHENOUT','PATOUT','PLASOUT',
     +           'STEPS','PLOT'/

      IFILE=12
      JFILE=13
      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME
      IF (NOCOEF .EQ. 1) THEN
         WRITE (6,300)
         RETURN
      ENDIF
      ZADD(1)=0
      ZADD(2)=0
      ZADD(3)=0
      NM1=NOTU-1
      NTAX=NOTU
      NP1=NOTU+1
      NT2=NOTU*2
      NT2M1=NT2-1
      NT2M2=NT2-2
      NT2M3=NT2-3
      NSTOP=0

*  Defaults:

      INTVL(1)=0
      INTVL(2)=3
      INTVL(3)=1
      INTVL(4)=5
      INTVL(5)=1
      REALVL(1)=0.
      DO 500 I=1,7
         LOGVL(I)=.FALSE.
500   CONTINUE

      CALL READER (5,1,7,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) RETURN
      NCOEF=INTVL(1)
      NADD=INTVL(2)
      NFIT=INTVL(3)
      MAXTR=INTVL(4)
      NROOT=INTVL(5)
      YLIM=REALVL(1)
      NONEG=LOGVL(1)
      NOOPT=LOGVL(2)
      MATOUT=LOGVL(3)
      PATOUT=LOGVL(4)
      NPLASY=LOGVL(5)
      STEPS=LOGVL(6)
      PLOT=LOGVL(7)

*  Error check and interpretation of parameters:

      IERROR=0
      IF (NCOEF.GT.7 .AND. NCOEF.LT.14) THEN
         WRITE (6,201) COEF(NCOEF)
      ELSE
         WRITE (6,301) INTST(1),NCOEF
         IERROR=1
      ENDIF
      IF (NADD .EQ. 0) THEN
         WRITE (6,202)
      ELSE IF (NADD.GE.1 .AND. NADD.LE.3) THEN
         WRITE (6,203) NADD
      ELSE
         WRITE (6,301) INTST(2),NADD
         IERROR=1
      ENDIF
      IF (NADD .EQ. 0) THEN
         IF (NFIT .EQ. 1) THEN
            WRITE (6,204)
         ELSE IF (NFIT .EQ. 2) THEN
            WRITE (6,205)
         ELSE IF (NFIT .EQ. 3) THEN
            WRITE (6,206)
         ELSE
            WRITE (6,301) INTST(3),NFIT
            IERROR=1
         ENDIF

         IF (MAXTR.LE.0 .OR. MAXTR.GE.31) THEN
            WRITE (6,301) INTST(4),MAXTR
            IERROR=1
         ELSE
            WRITE (6,207) MAXTR
         ENDIF
      ENDIF
      IF (NONEG) WRITE (6,208)
      IF (NROOT .EQ. 1) THEN
         WRITE (6,209)
      ELSE IF (NROOT .EQ. 2) THEN
         WRITE (6,210)
      ELSE
         WRITE (6,301) INTST(5),NROOT
         IERROR=1
      ENDIF
      IF (IERROR.EQ.1) RETURN
      IF (NOOPT) WRITE (6,211)

*  Get phenetic distance matrix:

      REWIND 15
      DO 501 J=1,NMAT
         IF (MATFIL(J).EQ.NCOEF) GO TO 16
            READ (15)
501   CONTINUE
      WRITE (6,302) NCOEF,COEF(NCOEF)
      RETURN
 16   READ (15) ((DPHEN(I,J),I=J,NOTU),J=1,NOTU)
      REWIND 15
      DO 502 I=1,NOTU
         DPHEN(I,I)=0.
502   CONTINUE
      IF (MATOUT) CALL MATPRT (5,0,0,'DUMMY',DPHEN,POPNAM,1,MT)

      IF (NADD .EQ. 0) THEN
         NCRIT=3
         SICRT=.FALSE.
      ELSE
         NCRIT=1
         ICRIT=NADD
         SICRT=.TRUE.
         NEWNOD=NOTU
      ENDIF
      NODE=NOTU

*  Find smallest phenetic distance for initial branch:

      DMIN=99.E9
      D(1)=0.
      ANC(1)=-999
      DO 503 I=2,NOTU
         ANC(I)=-999
         DO 504 J=1,I-1
            D(MDEX(I,I))=0.
            IF (DPHEN(I,J).LT.DMIN) THEN
               DMIN=DPHEN(I,J)
               X=J
               Y=I
            ENDIF
*           (Copy original distance matrix to D)
            D(MDEX(I,J))=DPHEN(I,J)
504      CONTINUE
503   CONTINUE
      ANC(X)=Y
      ANC(Y)=0
      HYPANC=Y
      IF (.NOT. SICRT) THEN
         OPEN (12,ACCESS='DIRECT',STATUS='SCRATCH',RECL=DWRECL,
     +      FORM='UNFORMATTED')
         OPEN (13,ACCESS='DIRECT',STATUS='SCRATCH',RECL=DWRECL,
     +      FORM='UNFORMATTED')
         WRITE (IFILE,REC=1) ANC,D
         NEWNOD=NODE
      ENDIF
      NUMTR=1

*  Select OTU to add next and to which branch it will add:

 21   NUMTRS=NUMTR
      NUMTR=0
      OLDNOD=NEWNOD
      IF (SICRT) NUMTRS=1
      DO 505 NTREE=1,NUMTRS
         NUMTR=NUMTR+1
         DO 506 NC=1,NCRIT
            IF (.NOT. SICRT) THEN
               ICRIT=NC
               NODE=OLDNOD
               READ (IFILE,REC=NTREE) ANC,D
            ENDIF
            M=99.E9
            DMIN=99.E9
            DO 507 I=1,NOTU
               IF (ANC(I) .EQ. -999) THEN
                  MINDB=99.E9

*                 Calculate distance from each unplaced OTU to each br
*                 branch of network:

                  DO 508 K=1,NODE
                     IF (ANC(K).GT.0) THEN
                        DBRNCH=(D(MDEX(I,K))
     +                         + D(MDEX(I,ANC(K)))
     +                         - D(MDEX(K,ANC(K))))/2.
                        IF (DBRNCH.LT.MINDB) THEN
                           BCON=K
                           MINDB=DBRNCH
                           ITIE=0
                        ELSE IF (DBRNCH.EQ.MINDB) THEN
                           ITIE=1
                        ENDIF
                     ENDIF
508               CONTINUE

*                 Select an OTU to add next using one of the three
*                 addition criteria:

                  BR1=BCON
                  BR2=ANC(BCON)
                  D1=D(MDEX(I,BR1))
                  D2=D(MDEX(I,BR2))
                  D3=D(MDEX(BR1,BR2))

                  IF (ICRIT.EQ.1) THEN
*                    Criterion I:
                        MI=MIN(2.*MINDB, D1+D3-D2, D2+D3-D1)

                  ELSE IF (ICRIT.EQ.2) THEN
*                    Criterion II:
                        MI=MIN(D1,D2,D3)

                  ELSE IF (ICRIT.EQ.3) THEN
*                    Criterion III:
                        MI=MINDB
                  ENDIF

                  IF ((MI.LT.M).OR.((MI.EQ.M).AND.(MINDB.LT.DMIN))) THEN
                     M=MI
                     X=BR1
                     Y=BR2
                     Z=I
                     DMIN=MINDB
                     KSTART=BCON+1
                     NTIE=0
                     IF (ITIE.EQ.1) NTIE=1
                     ITIE=0
                  ENDIF
               ENDIF
507         CONTINUE

            IF (.NOT. SICRT) THEN

*              Determine if OTU selected for next addition is identical
*              to that specified by a previous criterion; if so, go on
*              to next criterion (MAC approach):

               ZADD(ICRIT)=Z
               IF (ICRIT.NE.1) THEN
                  DO 509 I=1,ICRIT-1
                     IF (Z.EQ.ZADD(I)) GO TO 506
509               CONTINUE
                  NUMTR=NUMTR+1
               ENDIF
            ENDIF
 31         CONTINUE

*        Calculate new branch lengths:

            DNZ=DMIN
            DNX=D(MDEX(X,Z))-DNZ
            DNY=D(MDEX(Y,Z))-DNZ

            IF (NONEG) THEN
*              Restriction of non-negative branch lengths:
               IF (DNX.LT.0.) THEN
                  DNY=DNY+DNX
                  DNZ=DNZ-DNX
                  DNX=0.
               ENDIF
               IF (DNY.LT.0.) THEN
                  DNX=DNX+DNY
                  DNZ=DNZ-DNY
                  DNY=0.
               ENDIF
               IF (DNZ.LT.0.) DNZ=0.
            ENDIF

            CALL BRANCH (X,Y,Z,NODE,DNX,DNY,DNZ,ANC,P,NOTU,MHLFMT,MT2)
            NEWNOD=NODE

            IF (.NOT. SICRT) THEN
*              Determine which partial networks are to be held for next
*              step (MAC):
               NTIDEN=0
*              (Check if network topology identical to a previous one)
               IF (NUMTR .NE. 1) CALL TOPOL (NUMTR,NTIDEN,JFILE,
     +                                       NODE,ANC,ANCTOP,ANCPRV,MT2)
               TEST=GFTEST(P,DPHEN,ANC,MHLFMT,MT,MT2)
               IF (STEPS) CALL TMPTRE (NODE,TEST,ANC,P,X,Y,Z,
     +                                 NTREE,MHLFMT,MT2)
               IF (NTIDEN.NE.0) THEN
                  NUMTR=NUMTR-1

*                 If network with same topology has closer fit, replace
*                 previous network; otherwise discard new network:

                  IF (TEST.LT.FITNS(NTIDEN)) THEN
                     FITNS(NTIDEN)=TEST
                     CALL DCALC (D,P,ANC,NODE,Z,NOTU,MHLFMT,MT2)
                     WRITE (JFILE,REC=NTIDEN) ANC,D
                  ENDIF

*              Unique network:

               ELSE IF (NUMTR.LE.MAXTR) THEN
*                 ( Less than MAXTR partial networks have been saved; )
*                 ( automatically save this partial network           )
                  CALL DCALC (D,P,ANC,NODE,Z,NOTU,MHLFMT,MT2)
                  WRITE (JFILE,REC=NUMTR) ANC,D
                  FITNS(NUMTR)=TEST

               ELSE
*                 ( MAXTR partial networks already on file; check to )
*                 ( see if new partial network is superior to one    )
*                 ( already on file                                  )
                  NUMTR=NUMTR-1
                  MAXFIT=1
                  DO 510 I=2,NUMTR
                     IF (FITNS(I).GT.FITNS(MAXFIT)) MAXFIT=I
510               CONTINUE

                  IF (TEST.LT.FITNS(MAXFIT)) THEN
*                    ( Network has closer fit; will replace )
*                    ( previous network with poorest fit    )
                     FITNS(MAXFIT)=TEST
                     CALL DCALC (D,P,ANC,NODE,Z,NOTU,MHLFMT,MT2)
                     WRITE (JFILE,REC=MAXFIT) ANC,D
                  ENDIF
               ENDIF

               IF (NTIE.NE.0) THEN

*                 ( Tie occurred in selecting closest branch; try all )
*                 ( branches to which OTU Z is equidistant:           )

                  READ (IFILE,REC=NTREE) ANC,D
                  NTIE=0
                  NODE=OLDNOD
                  NTIEFD=0
                  KBEG=KSTART
                  DO 511 K=KBEG,NODE
                     IF (ANC(K).GT.0) THEN
                        IF ((D(MDEX(Z,K)) + D(MDEX(Z,ANC(K)))
     +                       - D(MDEX(K,ANC(K))))/2..LE.DMIN) THEN
                           NTIEFD=NTIEFD+1
                           IF (NTIEFD.LE.1) THEN
                              CALL SWITCH (K,ANC(K),Y,X)
                              KSTART=K+1
                              NUMTR=NUMTR+1
                              IF (STEPS) WRITE (6,212) Z,X,Y
                           ELSE
                              NTIE=1
                           ENDIF
                        ENDIF
                     ENDIF
511               CONTINUE
                  GO TO 31
               ENDIF
            ELSE

*              Single addition criterion:

               CALL DCALC (D,P,ANC,NODE,Z,NOTU,MHLFMT,MT2)
               IF (STEPS) CALL TMPTRE (NODE,TEST,ANC,P,X,Y,Z,NTREE,
     +                                 MHLFMT,MT2)
            ENDIF
506      CONTINUE
505   CONTINUE
      IF (.NOT. SICRT) THEN
         CALL CHANGE (IFILE,JFILE)
         IF (STEPS) THEN
            WRITE (6,213)
            DO 512 I=1,NUMTR
               READ (IFILE,REC=I) ANC,D
               IF (NFIT.EQ.1) THEN
                  WRITE (6,214) I,FITNS(I)
               ELSE IF (NFIT.EQ.2) THEN
                  WRITE (6,215) I,FITNS(I)
               ELSE
                  R= -FITNS(I)
                  WRITE (6,216) I,R
               ENDIF
               DO 513 J=1,NEWNOD
                  IF (ANC(J).GT.0) WRITE (6,217) J,ANC(J)
513            CONTINUE
512         CONTINUE
         ENDIF
      ENDIF
      IF (NEWNOD.LT.NT2M2) GO TO 21

      IF (SICRT) NUMTR=1
      DO 514 NTREE=1,NUMTR
         IF (.NOT.SICRT) READ (IFILE,REC=NTREE) ANC,D
         NSTOP=0
         CALL CONGEN (ANC,NCONN,MT2)
         CALL NETWRK (P,HPLASY,DPHEN,ANC,POPNAM,TERMB,PDSUM,
     +                MHLFMT,MT,MT2)
         CALL TREE (PDSUM,P,NCONN,NTERM,POPNAM,TERMB,ANC,IGRP,LINKED,
     +      ITAX,CD,CUSE,IVECT,ICLINE,ICBEG,ICEND,NPOS,NCENT,NLINE,
     +      NORDER,MT,MT2,MHLFMT)

         IF (.NOT. NOOPT) THEN

*           Optimization calls:

            NSTOP=1
            DO 515 I=1,NOTU
               DO 516 J=1,I
                  D(MDEX(I,J))=DPHEN(I,J)
516            CONTINUE
515         CONTINUE
            CALL DISOPT (D,P,ANC,NTERM,NWT,NCONN,MHLFMT,MT2)
            CALL NETWRK (P,HPLASY,DPHEN,ANC,POPNAM,TERMB,PDSUM,
     +         MHLFMT,MT,MT2)
            CALL TREE (PDSUM,P,NCONN,NTERM,POPNAM,TERMB,ANC,IGRP,LINKED,
     +         ITAX,CD,CUSE,IVECT,ICLINE,ICBEG,ICEND,NPOS,NCENT,NLINE,
     +         NORDER,MT,MT2,MHLFMT)
         ENDIF
514   CONTINUE
      RETURN

200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Distance Wagner Procedure',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,

     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')//////)
201   FORMAT (/' Coefficient used: ',A48)
202   FORMAT (' Multiple addition criterion procedure of Swofford ',
     +'(1981) will be used')
203   FORMAT (' Criterion',I2,' (See Swofford, 1981) will be used to ',
     +'determine addition sequence')
204   FORMAT (' Prager and Wilson''s F-value will be used in selecting',
     +' partial networks')
205   FORMAT (' Percent standard deviation will be used in selecting ',
     +'partial networks')
206   FORMAT (' Cophenetic correlation will be used in selecting ',
     +'partial networks')
207   FORMAT (' Number of networks to be held for successive steps =',
     +I3)
208   FORMAT (' Restriction of non-negative branch lengths imposed')
209   FORMAT (' Tree will be rooted at midpoint of greatest patristic ',
     +'distance')
210   FORMAT (' Tree will be rooted using outgroup method')
211   FORMAT (' Branch length optimization suppressed')
212   FORMAT (/' Tie found in selecting closest branch for OTU',
     +I3/' branch',I3,'-',I3,' is as close as a previously-selected ',
     +'branch')
213   FORMAT (///' Networks saved for further operations:'/)
214   FORMAT(' Network number',I2,' (F=',F11.6,')')
215   FORMAT(' Network number',I2,' (SD=',F11.6,')')
216   FORMAT(' Network number',I2,' (r=',F8.6,')')
217   FORMAT(2I5)
300   FORMAT (' ERROR: Step SIMDIS has not been performed')
301   FORMAT (/' ERROR: Illegal value for ',A7,':',I5)
302   FORMAT (////' ERROR: Matrix no.',i3,' = ',A48/8X,
     +'not on file.')
      END


      SUBROUTINE DCALC (D,P,ANC,NODE,Z,NOTU,MHLFMT,MT2)

*  Estimates phenetic difference between new node and all OTU's
*  not yet on tree

      REAL D,P
      INTEGER ANC,Z
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION D(MHLFMT),P(MHLFMT),ANC(MT2)

      NODEM1=NODE-1
      DO 500 I=1,NOTU
         IF (ANC(I) .EQ. -999) THEN
            DMAX=0.
            DO 501 J=1,NODEM1
               IF (I.NE.J .AND. ANC(J).NE.-999) THEN
                  DINODE=D(MDEX(I,J))-P(MDEX(NODE,J))
                  IF (DINODE.GE.DMAX) DMAX=DINODE
               ENDIF
501         CONTINUE
            D(MDEX(NODE,I))=DMAX
            DO 502 J=NP1,NODEM1
               DCHECK=D(MDEX(I,Z))-P(MDEX(J,Z))
               IF(DCHECK.GT.D(MDEX(I,J))) D(MDEX(I,J))=DCHECK
502         CONTINUE
         ENDIF
500   CONTINUE
      RETURN
      END


      FUNCTION GFTEST (P,DPHEN,ANC,MHLFMT,MT,MT2)

*  Used in evaluation of partial networks in DISWAG (MAC approach).

*  Value returned is Prager and Wilson F (NFIT=1), Fitch and Margoliash
*  percent standard deviation (NFIT=2), or negative correlation
*  coefficient (NFIT=3)

      REAL P,DPHEN
      INTEGER ANC
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM

      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION P(MHLFMT),DPHEN(MT,MT),ANC(MT2)

      SSX1=0.
      SSX2=0.
      SX1X2=0.
      NCORR=0
      SUMX=0.
      SUMY=0.
      FARRF=0.
      FMSS=0.
      DO 500 I=2,NOTU
         IF (ANC(I).NE.-999) THEN
            DO 501 J=1,I-1
               IF (ANC(J).NE.-999) THEN
                  NCORR=NCORR+1
                  DIJ=DPHEN(I,J)
                  PIJ=P(MDEX(I,J))
                  SSX1=SSX1+DIJ*DIJ
                  SSX2=SSX2+PIJ*PIJ
                  SX1X2=SX1X2+DIJ*PIJ
                  SUMX=SUMX+DIJ
                  SUMY=SUMY+PIJ
                  FARRF=FARRF+ABS(PIJ-DIJ)
                  IF (DIJ .NE. 0.0) FMSS=FMSS+((DIJ-PIJ)*100./DIJ)**2
               ENDIF
501         CONTINUE
         ENDIF
500   CONTINUE
      SUMXSQ=SSX1-SUMX**2/NCORR
      SUMYSQ=SSX2-SUMY**2/NCORR
      COV=SX1X2-SUMX*SUMY/NCORR
      IF (SUMX.EQ.0. .OR. NCORR.EQ.3) THEN
         IF (NFIT.NE.3) THEN
            GFTEST=0.
         ELSE
            GFTEST=-1.
         ENDIF
         RETURN
      ENDIF
      IF (NFIT.LT.2) THEN
*        ( Prager and Wilson F )
         GFTEST=FARRF*100./SUMX
      ELSE IF (NFIT.EQ.2) THEN
*        ( Percent S.D. )
         GFTEST=SQRT(FMSS/(NCORR-1))
      ELSE
*        ( Cophenetic correlation )
         GFTEST=-COV/SQRT(SUMXSQ*SUMYSQ)
      ENDIF
      RETURN
      END


      SUBROUTINE TOPOL
     +   (NUMTR,NTIDEN,JFILE,NODE,ANC,ANCTOP,ANCPRV,MT2)

*  Checks two partial networks to see if they have the same topology

      INTEGER ANC,ANCTOP,ANCPRV
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION ANC(MT2),ANCTOP(MT2),ANCPRV(MT2)

      NSTD=0
      DO 500 NTR=1,NUMTR-1
*        ( Get ancestor function from a previously generated network )
         READ (JFILE,REC=NTR) ANCPRV
*        ( In most cases, networks will be distinguishable simply on )
*        ( basis of OTU's attached:                                  )
         DO 501 I=1,NTAX
            IF (ANC(I)*ANCPRV(I).LT.0) GO TO 500
501      CONTINUE
         IF (NODE.EQ.NTAX+1) GO TO 2

*           If cycle completely through 501 loop, networks have same
*           complement of OTU's; put ancestor functions into "standard
*           form" and check topologies:

            IF (NSTD.NE.1) THEN
               NSTD=1
*              ( Copy ANC to temporary array )
               DO 502 K=1,NODE
                  ANCTOP(K)=ANC(K)
502            CONTINUE
               CALL STAND (ANCTOP,NODE,MT2)
            ENDIF
            CALL STAND (ANCPRV,NODE,MT2)
            DO 503 K=1,NODE
               IF (ANCTOP(K) .NE. ANCPRV(K)) GO TO 500
503         CONTINUE

*        IF cycle completely through 503 loop, networks are identical

  2      NTIDEN=NTR
         RETURN
500   CONTINUE

*  If cycle completely through 500 loop, new network is unique

      NTIDEN=0
      RETURN
      END


      SUBROUTINE STAND (ANC,NODE,MT2)

*  Puts ancestor function into "standard form" for comparison of
*  tree topologies

      INTEGER ANC,HYPANC,ANC1,ANC2
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION ANC(MT2)

      ICURR=NTAX
      DO 500 I=1,NTAX
         IF (ANC(I).GT.0) THEN
            NCODED=I+1000
  1         ITEMP=ANC(NCODED-1000)
               IF (ITEMP.LT.1000 .AND. ITEMP.NE.HYPANC) THEN
                  ICURR=ICURR+1
                  NCODED=ICURR+1000
                  ANC1=ANC(ITEMP)
                  IF (ANC1.EQ.ICURR) ANC1=ITEMP
                  ANC2=ANC(ICURR)
                  IF (ANC2.EQ.ITEMP) ANC2=NCODED
                  CALL ASUB (ITEMP,NCODED,ANC,NODE,MT2)
                  IF (ICURR .NE. ITEMP)
     +               CALL ASUB (ICURR,ITEMP,ANC,NODE,MT2)
                  ANC(ITEMP)=ANC2
                  ANC(ICURR)=ANC1
                  GO TO 1
               ENDIF
         ENDIF
500   CONTINUE

*  Decode:

      DO 501 I=1,NODE
         IANC=ANC(I)
         IF (IANC.GT.1000) ANC(I)=IANC-1000
501   CONTINUE
      RETURN
      END


      SUBROUTINE ASUB (IVAL,JVAL,ANC,NODE,MT2)

      INTEGER ANC
      DIMENSION ANC(MT2)

      DO 500 I=1,NODE
         IF (ANC(I).EQ.IVAL) ANC(I)=JVAL
500   CONTINUE
      RETURN
      END


      SUBROUTINE BRANCH
     +   (X,Y,Z,NODE,DNX,DNY,DNZ,ANC,P,NOTU,MHLFMT,MT2)

*  Updates patristic distance matrix and ancestor function

      IMPLICIT INTEGER(B,X-Z)
      REAL P
      INTEGER ANC,HYPANC
      LOGICAL XISANC,XSIDE
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      DIMENSION P(MHLFMT),ANC(MT2)

      NODE=NODE+1
      P(MDEX(NODE,NODE))=0.

      XISANC=ANC(Y).EQ.X
      IF (.NOT. XISANC) THEN
         ANC(X)=NODE
         ANC(NODE)=Y
         ANC(Z)=NODE
      ELSE
         ANC(Y)=NODE
         ANC(NODE)=X
         ANC(Z)=NODE
      ENDIF
      P(MDEX(NODE,X))=DNX
      P(MDEX(NODE,Y))=DNY
      P(MDEX(NODE,Z))=DNZ
      P(MDEX(X,Z))=DNX+DNZ
      P(MDEX(Y,Z))=DNY+DNZ
      P(MDEX(X,Y))=DNX+DNY
      IF (NODE.EQ.NOTU+1) RETURN
      DO 500 I=1,NODE-1
         IF (ANC(I).NE.-999) THEN
            IF (I.NE.X .AND. I.NE.Y .AND. I.NE.Z) THEN
               IANC=ANC(I)
  3            IF (IANC.NE.X .AND. IANC.NE.Y .AND. IANC.NE.0) THEN
                  IANC=ANC(IANC)
                  GO TO 3
               ENDIF
               IF (IANC .EQ. 0) THEN
                  XSIDE=XISANC
               ELSE
                  XSIDE=IANC.EQ.X
               ENDIF
               IF (XSIDE) THEN
                  PINODE=P(MDEX(I,X))+DNX
               ELSE
                  PINODE=P(MDEX(I,Y))+DNY
               ENDIF
               P(MDEX(NODE,I))=PINODE
               P(MDEX(I,Z))=PINODE+DNZ
            ENDIF
         ENDIF
500   CONTINUE
      RETURN
      END


      SUBROUTINE TMPTRE
     +   (NODE,TEST,ANC,P,X,Y,Z,NTREE,MHLFMT,MT2)

*  Prints partial network (STEPS option)

      IMPLICIT INTEGER (B,X-Z)
      REAL P
      INTEGER ANC
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      DIMENSION P(MHLFMT),ANC(MT2)

      IF (NADD.NE.0) THEN
         NUMSTP=NODE-NOTU
         WRITE (6,200) NUMSTP
         WRITE (6,201) Z,X,Y
      ELSE
         IF (NFIT.LT.2) THEN
            WRITE (6,202) TEST
         ELSE IF (NFIT.EQ.2) THEN
            WRITE (6,203) TEST
         ELSE
            R=-TEST
            WRITE (6,204) R
         ENDIF
         WRITE (6,205) Z,X,Y,NTREE
      ENDIF

      DO 500 I=1,NODE
         IF (ANC(I).GT.0) THEN
            CALL SWITCH (I,ANC(I),BR2,BR1)
            WRITE (6,206) BR1,BR2,P(MDEX(BR1,BR2))
         ENDIF
500   CONTINUE
      RETURN

200   FORMAT (///' Step',I3,':'/)
201   FORMAT (' OTU',I3,' added to',I3,'-',I3,' branch'/)
202   FORMAT(///' Partial network (F=',F11.6,')'/)
203   FORMAT (///' Partial network (SD=',F11.6,')'/)
204   FORMAT(///' Partial network (r=',F8.6,')'/)
205   FORMAT (' OTU',I3,' added to',I3,'-',I3,' branch of tree no.',
     +I2/)
206   FORMAT (2I5,F10.3)
      END


      SUBROUTINE CONGEN (ANC,NCONN,MT2)

*  Generates connection function from branch stack

      IMPLICIT INTEGER (B)
      INTEGER ANC,NCONN,HYPANC
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION ANC(MT2),NCONN(MT2,3)

      DO 500 I=1,NT2M2
         NCONN(I,1)=ANC(I)
500   CONTINUE
      DO 501 I=NP1,NT2M2
         IF (ANC(I) .EQ. HYPANC) THEN
            NCONN(HYPANC,1)=I
            GO TO 1
         ENDIF
501   CONTINUE

  1   DO 502 I=NP1,NT2M2
         JSTART=1
         K=2

  2      JBEG=JSTART
         DO 503 J=JBEG,NT2M2
            IF (ANC(J) .EQ. I) THEN
               NCONN(I,K)=J
               IF (K.EQ.3) GO TO 502
                  K=3
                  JSTART=J+1
                  GO TO 2
            ENDIF
503      CONTINUE
502   CONTINUE
      RETURN
      END


      SUBROUTINE NETWRK
     +   (P,HPLASY,DPHEN,ANC,POPNAM,TERMB,PDSUM,MHLFMT,MT,MT2)

*  Outputs network topology and branch lengths

      IMPLICIT INTEGER (B,X-Z)
      CHARACTER POPNAM*16
      REAL P,HPLASY,DPHEN,TERMB
      INTEGER ANC
      LOGICAL PATOUT,NPLASY
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION POPNAM(MT),P(MHLFMT),HPLASY(MT,MT),DPHEN(MT,MT),
     +   TERMB(MT),ANC(MT2)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP

      CALL PAGSKP
      WRITE (6,200)
      IF (NSTOP.EQ.1) WRITE (6,201)
      WRITE (6,202)
      PDSUM=0.
      DO 500 K=1,NT2M2
         IF (ANC(K).NE.0) THEN
            CALL SWITCH (K,ANC(K),BR2,BR1)
            SEGLEN=P(MDEX(BR1,BR2))
            IF (BR1.LE.NOTU) THEN
               WRITE (6,203) POPNAM(BR1),BR2,SEGLEN
               TERMB(BR1)=SEGLEN
            ELSE
               WRITE (6,204) BR1,BR2,SEGLEN
            ENDIF
            PDSUM=PDSUM+SEGLEN
         ENDIF
500   CONTINUE
      WRITE (6,205)

*  Print patristic distance matrix:

      IF (PATOUT) THEN
*        Copy patristic distances to HPLASY (for printing only)
         DO 501 I=1,NOTU
            DO 502 J=1,I
               HPLASY(J,I)=P(MDEX(I,J))
502         CONTINUE
501      CONTINUE
         CALL MATPRT (6,0,0,'DUMMY',HPLASY,POPNAM,2,MT)
      ENDIF

      CALL FIT (P,DPHEN,HPLASY,POPNAM,NPLASY,MHLFMT,MT)
      RETURN

200   FORMAT (///' Branch lengths and linkages for unrooted Wagner ',
     +'network'/1X,55('*'))
201   FORMAT (//' (After optimization)')
202   FORMAT (///T11,45('-')/T21,'Linkage'/T11,27('-')/T11,
     +'OTU or HTU',14X,'HTU',T43,'Branch length'/T11,45('-'))
203   FORMAT (T11,A16,T35,I3,F13.3)
204   FORMAT (T11,I3,T35,I3,F13.3)
205   FORMAT (T11,45('-'))
      END


      SUBROUTINE DISOPT (D,P,ANC,NTERM,NWT,NCONN,MHLFMT,MT2)

*  Tree traversal routine for "outside-in" optimization of distance
*  Wagner networks. actual branch-length optimization is done in bcalc.

      IMPLICIT INTEGER (B)
      REAL D,P
      INTEGER NTERM,NWT,NCONN,ANC,OLD,NEW
      PARAMETER (OLD=666666, NEW=777777)
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION D(MHLFMT),P(MHLFMT),NTERM(MT2),NWT(MT2),NCONN(MT2,3),
     +   ANC(MT2)

      DO 500 I=1,NOTU
         NTERM(I)=1
         NWT(I)=1
500   CONTINUE
      DO 501 I=NP1,NT2M2
         NTERM(I)=0
         NWT(I)=1
501   CONTINUE
      NBRNCH=NT2M3
  1   BRLESS=0
      DO 502 I=1,NT2M2
         IF (NTERM(I) .EQ. 1) THEN
            DO 503 J=1,3
               INODE=NCONN(I,J)
               IF (NTERM(INODE).NE.2) THEN
                  ITERM1=I
                  DO 504 K=1,3
                     NCK=NCONN(INODE,K)
                     IF (NTERM(NCK).EQ.1 .AND. NCK.NE.ITERM1) THEN
                        ITERM2=NCK
                        GO TO 2
                     ENDIF
504               CONTINUE

*                 ( If cycle completely through 504 loop, no suitable )
*                 ( ITERM2 exists                                     )
                  GO TO 502
               ENDIF
503         CONTINUE
            GO TO 502

  2         NTERM(ITERM1)=OLD
            NTERM(ITERM2)=OLD
            BRLESS=BRLESS+2
            CALL BCALC (ITERM1,ITERM2,INODE,D,P,NTERM,NWT,MHLFMT,MT2)
            NTERM(INODE)=NEW
            IF (NBRNCH.EQ.3) GO TO 3
         ENDIF
502   CONTINUE

*  Mark previously generated "INODE's" as terminal and "TERM's" as
*  non-terminal

  3   DO 505 I=1,NT2M2
         IF (NTERM(I).EQ.NEW) NTERM(I)=1
         IF (NTERM(I).EQ.OLD) NTERM(I)=2
505   CONTINUE
      NBRNCH=NBRNCH-BRLESS
      IF (NBRNCH.GT.1) GO TO 1

*  Calculate new patristic distance matrix

      CALL PDCALC (P,ANC,MT2,MHLFMT)
      RETURN
      END


      SUBROUTINE BCALC (ITERM1,ITERM2,INODE,D,P,NTERM,NWT,MHLFMT,MT2)

*  Calculates optimized branch lengths using method of Swofford (1981)

      REAL D,P
      INTEGER NTERM,NWT,OLD
      LOGICAL NONEG
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION D(MHLFMT),P(MHLFMT),NTERM(MT2),NWT(MT2)
      DATA OLD/666666/,NEW/777777/

      WTSUM=0.
      PAT1=0.
      PAT2=0.
      K=0
      DO 500 I=1,NT2M2
         IF (NTERM(I) .EQ. 1 .OR. NTERM(I) .EQ. OLD) THEN
            IF (I.NE.ITERM1 .AND. I.NE.ITERM2) THEN
               K=K+1
               DT1T2=D(MDEX(ITERM1,ITERM2))
               DINODE=(D(MDEX(I,ITERM1))+D(MDEX(I,ITERM2))-DT1T2)/2.
               D(MDEX(I,INODE))=DINODE
               WT=NWT(I)*NWT(ITERM1)*NWT(ITERM2)
               WTSUM=WTSUM+WT
               DNT1=D(MDEX(I,ITERM1))-DINODE
               DNT2=D(MDEX(I,ITERM2))-DINODE
               IF (NONEG) THEN
                  IF (DNT1.LT.0.) THEN
                     DNT2=DNT2+DNT1
                     DNT1=0.
                  ELSE
                     IF (DNT2.LT.0.) THEN
                        DNT1=DNT1+DNT2
                        DNT2=0.
                     ENDIF
                  ENDIF
               ENDIF
               PAT1=PAT1+DNT1*WT
               PAT2=PAT2+DNT2*WT
            ENDIF
         ENDIF
500    CONTINUE

*  Estimate distances between newly created nodes

      DO 501 I=1,NT2M2
         IF (NTERM(I) .EQ. NEW) THEN
            D(MDEX(I,INODE))=(D(MDEX(I,ITERM1))
     +                        +D(MDEX(I,ITERM2))-DT1T2)/2.
         ENDIF
501   CONTINUE
      PNT1=PAT1/WTSUM
      PNT2=PAT2/WTSUM
      NWT(INODE)=NWT(ITERM1)+NWT(ITERM2)
      P(MDEX(INODE,ITERM1))=PNT1
      P(MDEX(INODE,ITERM2))=PNT2
      RETURN
      END


      SUBROUTINE PDCALC (P,ANC,MT2,MHLFMT)

*  Recalculates patristic distance matrix given branch lengths and
*  tree topology

      IMPLICIT INTEGER (B)
      REAL P
      INTEGER COMMAN,COMANC,HYPANC,ANC
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION P(MHLFMT),ANC(MT2)

*  Get p.d.'s for all pairs of taxa:

      P(1)=0.
      DO 500 ITX=2,NT2M2
         P(MDEX(ITX,ITX))=0.
         DO 501 JTX=1,ITX-1
            COMANC=COMMAN(ITX,JTX,ANC,HYPANC,MT2)
            PD=0.
            IANC=ITX
  1         IF (IANC.NE.COMANC) THEN
               I=IANC
               IANC=ANC(IANC)
               PD=PD+P(MDEX(I,IANC))
               GO TO 1
            ENDIF
            JANC=JTX
  2         IF (JANC.NE.COMANC) THEN
               J=JANC
               JANC=ANC(JANC)
               PD=PD+P(MDEX(J,JANC))
               GO TO 2
            ENDIF
            P(MDEX(ITX,JTX))=PD
501      CONTINUE
500   CONTINUE
      RETURN
      END


      SUBROUTINE TREE
     +   (PDSUM,P,NCONN,NTERM,POPNAM,TERMB,ANC,IGRP,LINKED,ITAX,CD,
     +    CUSE,IVECT,ICLINE,ICBEG,ICEND,NPOS,NCENT,NLINE,NORDER,MT,
     +    MT2,MHLFMT)

*  Places tree data into form used for input to TRPRIN and TRPLOT

      IMPLICIT INTEGER (B,X-Z)
      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,POPNAM*16
      REAL P,TERMB,CD,CUSE
      INTEGER NCONN,NTERM,ITAX,ANC,IGRP,LINKED,IVECT,ICLINE,ICBEG,
     +   ICEND,NPOS,NCENT,NLINE,NORDER,HYPANC
      LOGICAL PLOT
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION POPNAM(MT),P(MHLFMT),TERMB(MT),CD(MT),CUSE(MT),
     +   NCONN(MT2,3),NTERM(MT2),ITAX(MT,2),ANC(MT2),IGRP(MT),
     +   LINKED(MT),IVECT(101),ICLINE(MT,2),ICBEG(MT,2),ICEND(MT,2),
     +   NPOS(MT),NCENT(MT),NLINE(MT),NORDER(MT)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP

      NM2=NOTU-2
      MROOT=NT2M1
  1   IF (NROOT.NE.2) THEN
         CALL MIDPNT (BR1,BR2,PATHLF,P,TERMB,ANC,HYPANC,
     +                MT,MT2,MHLFMT)
      ELSE
         CALL OUTGRP (BR1,BR2,P,TERMB,NTERM,NCONN,IGRP,
     +                MT,MT2,MHLFMT)
         IF (NROOT.EQ.1) GO TO 1
      ENDIF

*  Collapse tree into (BR1,BR2) branch:

      DO  500 I=1,NOTU
         NTERM(I)=I
500   CONTINUE
      DO 501 I=NP1,NT2M1
         NTERM(I)=0
501   CONTINUE
      N=0
  4   DO 502 I=1,NT2M2
         IF (NTERM(I).GT.0 .AND. I.NE.BR1 .AND. I.NE.BR2) THEN
            DO 503 J=1,3
               INODE=NCONN(I,J)
               IF (NTERM(INODE).GE.0) THEN
                  ITERM1=I
                  DO 504 K=1,3
                     NCK=NCONN(INODE,K)
                     IF (NTERM(NCK).GT.0 .AND. NCK.NE.ITERM1) THEN
                        ITERM2=NCK
                        IF (ITERM2.NE.BR1 .AND. ITERM2.NE.BR2) GO TO 5
                     ENDIF
504               CONTINUE
                  GO TO 502
               ENDIF
503         CONTINUE
  5         N=N+1
            CALL SWITCH (NTERM(ITERM1),NTERM(ITERM2),NTERM2,NTERM1)
            NTERM(ITERM1)=-1
            NTERM(ITERM2)=-1
            NTERM(INODE)=NTERM1
            ITAX(N,1)=NTERM1
            ITAX(N,2)=NTERM2
            CD(N)=P(MDEX(MROOT,INODE))
            IF (N.EQ.NM2) GO TO 6
         ENDIF
502   CONTINUE
      IF (N.LT.NM1) GO TO 4
  6   CALL SWITCH (NTERM(BR1),NTERM(BR2),NTERM2,NTERM1)
      ITAX(NM1,1)=NTERM1
      ITAX(NM1,2)=NTERM2
      CD(NM1)=0.

*  Find most advanced OTU (for tree printing)

      IF (NROOT.NE.1) THEN
         PATHLF=0.
         DO 505 I=1,NOTU
            IF (P(MDEX(MROOT,I)).GT.PATHLF) PATHLF=P(MDEX(MROOT,I))
505      CONTINUE
      ENDIF
      CALL PAGSKP
      IF (NROOT.NE.2) THEN
         WRITE (6,200)
      ELSE
         WRITE (6,201)
      ENDIF
      IF (NSTOP.EQ.1) WRITE (6,202)
      CALL TRPRIN (1,1,PATHLF,PDSUM,LINKED,POPNAM,TERMB,ITAX,CD,CUSE,
     +   IVECT,ICLINE,ICBEG,ICEND,NPOS,NCENT,NLINE,NORDER,MT)
      WRITE (6,204)
      RETURN

200   FORMAT (///' Wagner tree produced by rooting at midpoint of ',
     +'longest path'/1X,59('*'))
201   FORMAT (///' Wagner tree rooted using outgroup method'/1X,40('*'))
202   FORMAT (//' (After optimization)')
204   FORMAT(//////)
      END


      SUBROUTINE MIDPNT
     +   (BR1,BR2,PATHLF,P,TERMB,ANC,HYPANC,MT,MT2,MHLFMT)

*  Roots tree by locating midpoint of longest path (patristic
*  distance) between two OTU's

      REAL P,TERMB
      INTEGER ANC,BR1,BR2,COMANC,COMMAN,HYPANC
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION P(MHLFMT),TERMB(MT),ANC(MT2)

      PATMAX=0.
      DO 500 I=2,NTAX
         DO 501 J=1,I-1
            IF (P(MDEX(I,J)).GT.PATMAX) THEN
               PATMAX=P(MDEX(I,J))
               LP1=J
               LP2=I
            ENDIF
501      CONTINUE
500   CONTINUE
      PATHLF=PATMAX/2.
      COMANC=COMMAN(LP1,LP2,ANC,HYPANC,MT2)
      D1=P(MDEX(COMANC,LP1))
      IF (D1.GE.PATHLF) THEN
         LPT=LP1
      ELSE
         LPT=LP2
      ENDIF
      IANC=LPT

  3   I=IANC
         IANC=ANC(I)
         IF (P(MDEX(IANC,LPT)).LT.PATHLF) GO TO 3
      BR1=I
      BR2=IANC
      PB1B2=P(MDEX(BR1,BR2))
      PMRB1=PATHLF-P(MDEX(BR1,LPT))
      PMRB2=PB1B2-PMRB1
      CALL PDROOT (BR1,BR2,PMRB1,PMRB2,P,TERMB,MT,MHLFMT)
      RETURN
      END


      SUBROUTINE OUTGRP
     +   (BR1,BR2,P,TERMB,NTERM,NCONN,IGRP,MT,MT2,MHLFMT)

*  Roots tree by locating node where specified outgroup taxa join
*  network

      REAL P,TERMB
      INTEGER NCONN,NTERM,IGRP,BR1,BR2
      COMMON/WAGPAR/NROOT,NONEG,NADD,NFIT,MAXTR,NOOPT,
     +   MATOUT,PATOUT,NPLASY,STEPS,PLOT,NALGOR,HYPANC,YLIM
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION P(MHLFMT),TERMB(MT),NCONN(MT2,3),NTERM(MT2),IGRP(MT)

*  Initialize NTERM and check to make sure at least one OTU specified
*  as outgroup

      IFD=0
      DO 500 I=1,NTAX
         NTERM(I)=IGRP(I)
         IF (NTERM(I).EQ.1) IFD=1
500   CONTINUE
      IF (IFD .EQ. 0) THEN
         WRITE (6,300)
         NROOT=1
         RETURN
      ENDIF
      DO 501 I=NP1,NT2M2
         NTERM(I)=0
501   CONTINUE

      NBRNCH=NT2M3
      NTHOLD=-1
  2   NTRMFD=0
      DO 502 I=1,NT2M2
         IF (NTERM(I) .EQ. 1) THEN
            NTRMFD=NTRMFD+1
            DO 503 J=1,3
               INODE=NCONN(I,J)
               IF (NTERM(INODE).NE.2) THEN
                  ITERM1=I
                  DO 504 K=1,3
                     NCK=NCONN(INODE,K)
                     IF (NTERM(NCK) .EQ. 1 .AND. NCK.NE.ITERM1) THEN
                        ITERM2=NCK
                        NTRMFD=NTRMFD+1
                        GO TO 3
                     ENDIF
504               CONTINUE
                  GO TO 502
               ENDIF
503         CONTINUE
            GO TO 502

  3         NTERM(ITERM1)=2
            NTERM(ITERM2)=2
            NTERM(INODE)=1
            NBRNCH=NBRNCH-2
            IF (NBRNCH.EQ.1) GO TO 5
         ENDIF
502   CONTINUE
      IF (NTRMFD.NE.1) THEN
         IF (NTRMFD .EQ. NTHOLD) THEN
            WRITE (6,301)
            NROOT=1
            RETURN
         ENDIF
         NTHOLD=NTRMFD
         IF (NBRNCH.GT.1) GO TO 2
      ENDIF
5     BR1=INODE
      BR2=ITERM1
      PB1B2=P(MDEX(BR1,BR2))
      PMAX1=0.
      PMAX2=0.
      DO 505 I=1,NTAX
         IF (IGRP(I).NE.1) THEN
            PIBR1=P(MDEX(I,BR1))
            IF (PIBR1.GT.PMAX1) PMAX1=PIBR1
         ELSE
            PIBR2=P(MDEX(I,BR2))
            IF (PIBR2.GT.PMAX2) PMAX2=PIBR2
         ENDIF
505   CONTINUE
      TOTAL=PMAX1+PMAX2+PB1B2
      HALF=TOTAL/2.
      IF (PMAX1.GE.HALF) THEN
*        ( Root near ingroup node, BR1 )
         PMRB1=.2*PB1B2
         PMRB2=PB1B2-PMRB1
      ELSE
         IF (PMAX2.GE.HALF) THEN
*           ( Root near outgroup node, BR2 )
            PMRB2=.2*PB1B2
            PMRB1=PB1B2-PMRB2
         ELSE
*           ( Root at midpoint between most divergent ingroup )
*           ( and outgroup OTU's                              )
            PMRB1=HALF-PMAX1
            PMRB2=PB1B2-PMRB1
         ENDIF
      ENDIF
      CALL PDROOT (BR1,BR2,PMRB1,PMRB2,P,TERMB,MT,MHLFMT)
      RETURN

300   FORMAT (///10X,'ERROR: No OTU(s) specified as outgroup with ',
     +'NROOT = 2./17X','Network will be rooted at midpoint.')
301   FORMAT (///10X,'WARNING: Specified ingroup is not monophyletic.'/
     +10X,'Network will be rooted at midpoint.')
      END


      INTEGER FUNCTION COMMAN (NODE1,NODE2,ANC,HYPANC,MT2)

*  Determines most recent common ancestor for any pair of nodes

      INTEGER ANC,HYPANC
      DIMENSION ANC(MT2)

      IF (NODE1 .EQ. HYPANC .OR. NODE2 .EQ. HYPANC) THEN
         COMMAN=HYPANC
         RETURN
      ELSE
         IANC=NODE1
  1      JANC=NODE2
  2      IF (JANC.NE.IANC) THEN
            IF (JANC .EQ. HYPANC) THEN
               IANC=ANC(IANC)
               GO TO 1
            ENDIF
            JANC=ANC(JANC)
            GO TO 2
         ENDIF
         COMMAN=IANC
      ENDIF
      RETURN
      END


      SUBROUTINE PDROOT
     +   (BR1,BR2,PMRB1,PMRB2,P,TERMB,MT,MHLFMT)

*  Calculates patristic distances from root to all other nodes
*  and adjusts TERMB (for printing purposes) if necessary

      REAL P,TERMB,PMRB1,PMRB2
      INTEGER BR1,BR2
      COMMON/VALS/NM1,NTAX,NP1,NT2,NT2M1,NT2M2,NT2M3,NSTOP
      DIMENSION P(MHLFMT),TERMB(MT)

      MROOT=NT2M1
      DO 500 I=1,NT2M2
         PIBR1=P(MDEX(I,BR1))
         PIBR2=P(MDEX(I,BR2))
         IF (PIBR1.LE.PIBR2) THEN
*           ( "I" is on "BR1" side )
            P(MDEX(MROOT,I))=PIBR1+PMRB1
         ELSE
*           ( "I" is on "BR2" side )
            P(MDEX(MROOT,I))=PIBR2+PMRB2
         ENDIF
500   CONTINUE
      IF (BR1.LE.NTAX) TERMB(BR1)=PMRB1
      IF (BR2.LE.NTAX) TERMB(BR2)=PMRB2
      RETURN
      END
