*=======================================================================*
*                                                                       *
*                        S T E P   F S T A T                            *
*                                                                       *
*      Non-hierarchical F-statistics from genotype frequency data       *
*                                                                       *
*  Calculates non-hierarchical F-statistics with gene and genotype      *
*  frequency data.  In the new version of FSTAT, calculations can be    *
*  made one of three ways.  The original FSTAT used unweighted mean     *
*  allele frequencies for the entire population.  This option can be    *
*  invoked with the UNWT option.  The default calculations follow       *
*  Nei (1977) and use weighted means.  The third option calculates      *
*  F-statistics using the method of Weir and Cockerham (1984).          *
*  See Black and Krafsur (1985) for an explanation of formulae.         *
*  Option:                                                              *
*     OUTPUT = 1  full output, including Fis(ik) values, F-statistics   *
*                    for each allele, and summary table of F-statistics *
*                    by locus                                           *
*            = 2  normal output (default). Fis(ik) values not printed.  *
*            = 3  minimal output. only summary table of F-statistics    *
*                    by locus printed.                                  *
*     UNWT   = F-statistics calculated using unweighted means.  This    *
*              is how the original BIOSYS calculated F-statistics.      *
*              This option cannot be used with Weir and Cockerham's     *
*              method.                                                  *
*     NEI    = F-statistics calculated using weighted means (default).  *
*              These calculations follow Nei (1977).                    *
*     WEIR   = F-statistics calculated using the method of              *
*              Weir and Cockerham (1985).                               *
*                                                                       *
*                                                                       *
*  Subprograms required by step FSTAT: FSTAT  READER  CONVRT            *
*                                                                       *
*=======================================================================*
*      PARAMETER (MXT=50, MXL=75, MXA=25, MXC=25, MXA2=MXA+MXA,
*    +   MXLT=MXL*MXT, MXT2=MXT+MXT, MXHLFT=MXT2*(MXT2+1)/2)

      SUBROUTINE FSTAT4
     +   (FREQ2,FREQ,FRQHOM,
     +    BIGP,SMALLP,SMPSQ,FISNUM,FISDEN,FISIK,FISI,
     +    FIS,FIT,FST,MINI,MAXI,MINI2,MAXI2,N,N2,NOBS,LOCNAM,PRES,VAR,
     +    MA,ML,MT,WGT,POPCNT,AL,BL,CL,CFJK,TJK,XFJK,FL,THETL,CAPFL,
     +    HETER)

      CHARACTER DATE*12,TIME*12,TITLE*72,FMT*80,LINE*80,
     +   INTST*64,REALST*8,LOGST*8,LOCNAM*5,FISROW*8,LETTER
      REAL FREQ2,FRQHOM,BIGP,SMALLP,SMPSQ,FISNUM,FISDEN,FISIK,FISI,FIS,
     +   FIT,FST,REALVL,TOTN,D2,A,B,C,WGT,POPCNT,AL,BL,CL,
     +    CFJK,TJK,XFJK,FL,THETL,CAPFL,HETER
      INTEGER MINI2,MAXI2,N2,NOBS,OUTPUT
      LOGICAL PRES,VAR,OUT1,OUT2,LOGVL,CRT,UNWT,WEIR,NEI
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/SYMB/LETTER(26)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION INTST(1),REALST(1),LOGST(4),LOCNAM(ML),
     +   FREQ2(MA,MT),FREQ(MA,ML),FRQHOM(MA,MT),BIGP(MA),SMALLP(MA),
     +   SMPSQ(MA),FISNUM(MT),FISDEN(MT),FISIK(MT),FISI(MT),FIS(ML),
     +   FIT(ML),FST(ML),MINI(ML),MAXI(ML),MINI2(MT),MAXI2(MT),N(ML),
     +   N2(MT),NOBS(MA,MA,ML),PRES(MA),VAR(ML),FISROW(15),INTVL(1),
     +   REALVL(1),LOGVL(4),WGT(MT),POPCNT(ML),AL(ML),BL(ML),CL(ML),
     +   CFJK(ML),TJK(ML),XFJK(ML),FL(ML),THETL(ML),CAPFL(ML),
     +   HETER(MA,ML)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA INTST/'OUTPUT'/,LOGST/'END','UNWT','NEI','WEIR'/
      DATA ZERO/1.E-6/,UNDEF/99.E9/
      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME
      IF (NDATYP .EQ. 3) THEN
         WRITE (6,300)
         RETURN
      ENDIF
      INTVL(1)=2
      LOGVL(1)=.FALSE.
      LOGVL(2)=.FALSE.
      LOGVL(3)=.TRUE.
      LOGVL(4)=.FALSE.
      CALL READER (1,0,4,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) RETURN
      OUTPUT=INTVL(1)
      NEI=LOGVL(3)
      UNWT=LOGVL(2)
      IF(UNWT) NEI=.FALSE. 
      WEIR=LOGVL(4)
      IF(WEIR) NEI=.TRUE.
      IF (OUTPUT.LE.0 .OR. OUTPUT.GE.4) THEN
         WRITE (6,301) OUTPUT
         RETURN
      ENDIF
      OUT1=.TRUE.
      OUT2=.TRUE.
      IF (OUTPUT.LT.2) THEN
         WRITE (6,201)
      ELSE IF (OUTPUT.EQ.2) THEN
         WRITE (6,202)
         OUT1=.FALSE.
      ELSE IF (OUTPUT.GT.2) THEN
         WRITE (6,203)
         OUT1=.FALSE.
         OUT2=.FALSE.
      ENDIF

      IF (UNWT) WRITE (6,226)
      IF (.NOT.UNWT) WRITE (6,225)
      IF (WEIR) WRITE (6,227)

      FISTOT=0.
      FITTOT=0.
      FSTTOT=0.
      DN1TOT=0.
      DN2TOT=0.
      POPA=0.
      POPB=0.
      POPC=0.
      REWIND 16
      REWIND 17
      REWIND 18

      IF(.NOT.WEIR)GO TO 551
      DO 609 J=1,NLOC
      DO 610 I1=1,NALL
         HETER(I1,J)=0.0
  610 CONTINUE
  609 CONTINUE

      DO 601 K=1,NOTU
         READ (18) NOBS
         READ (17) FREQ,MINI,MAXI,N
         DO 602 J=1,NLOC
          IF (MINI(J).NE.MAXI(J)) THEN
             MINIJ=MINI(J)
             MAXIJ=MAXI(J)
          ENDIF
      DO 603 I1=MINIJ,MAXIJ
         IF (FREQ(I1,J).EQ.0.) GO TO 603
      DO 604 I2=I1,MAXIJ
         IF (I2.LE.I1) GO TO 604
         IF (FREQ(I2,J).EQ.0.) GO TO 604
         OBS=NOBS(I1,I2,J)
         HETER(I1,J)=HETER(I1,J)+OBS
         HETER(I2,J)=HETER(I2,J)+OBS
  604 CONTINUE
  603 CONTINUE
  602 CONTINUE
  601 CONTINUE


  551 DO 500 J=1,NLOC
         TOTN=0.
         TOTNSQ=0.
         POPCNT(J)=NOTU
         READ (16) FREQ2,MINI2,MAXI2,N2,FRQHOM

         DO 600 I=1,NOTU
         TOTN=TOTN + N2(I)
         TOTNSQ=TOTNSQ+N2(I)**2.0
*    Calculate number of subpopulations per locus
         IF(N2(I).EQ.0) POPCNT(J)=POPCNT(J)-1                                
600      CONTINUE

*        Variance in Sample sizes for W&C Statistics.
           XNHAT=TOTN/POPCNT(J)                                    
           CSQ=(POPCNT(J)/((XNHAT**2.0)*(POPCNT(J)-1)))*((TOTNSQ
     +         /POPCNT(J))-(XNHAT**2.0))
           XNC=XNHAT*(1.0-(CSQ/POPCNT(J)))                         

*        Determine which alleles are present and catch invariant loci:

         NALLS=0
         VAR(J)=.TRUE.
         DO 501 K=1,NALL
            PRES(K)=.FALSE.
            BIGP(K)=0.
            SMALLP(K)=0.
            SMPSQ(K)=0.
            DO 502 I=1,NOTU
               IF (FREQ2(K,I).GT.ZERO) THEN
                  PRES(K)=.TRUE.
                  NALLS=NALLS+1
                  GO TO 501
               ENDIF
502         CONTINUE
501      CONTINUE
         IF (NALLS.LE.1) THEN
            VAR(J)=.FALSE.
            GO TO 500
         ENDIF

*        Fis(ik) values:

         DO 503 I=1,NOTU
            D2=N2(I)
            WGT(I)=D2/TOTN
            FISNUM(I)=0.
            FISDEN(I)=0.
503      CONTINUE
         IF (OUT2) CALL PAGSKP
         IF (.NOT. OUT1) THEN
            I1=1
            I2=NOTU
            GO TO 11
         ENDIF
         WRITE (6,205) LOCNAM(J)
         NCOL=15
         IF (CRT) NCOL=9
         I1=1
  9      I2=MIN(I1+NCOL-1,NOTU)
         NDIFF=NOTU-I2
         IF (NDIFF .EQ. 1) THEN
            I2=I2-1
            NCOL=NCOL-1
         ENDIF
         WRITE (6,206) ('--------',I=I1,I2)
         NCOLS=I2-I1+1
         NSKIPS=NCOLS-1
         FMT='(6X,  (4X),''Subpopulation'')'
         WRITE (FMT(5:6),'(I2)') NSKIPS
         WRITE (6,FMT)
         I1P1=I1+1
         WRITE (6,207) ('--------',I=I1P1,I2)

         WRITE (6,208) (I,I=I1,I2)
         WRITE (6,206) ('--------',I=I1,I2)
 11      DO 504 K=1,NALL
            IF (PRES(K)) THEN
               SPK=0.
               SPSM=0.
               SPSQR=0.
               DO 505 I=I1,I2
                  PCAP=FRQHOM(K,I)
                  PSMALL=FREQ2(K,I)
                  PSQR=PSMALL*PSMALL
                  PQ=PSMALL*(1.-PSMALL)
                  IF (UNWT) THEN
                     SPK=SPK+PCAP
                     SPSM=SPSM+PSMALL
                     SPSQR=SPSQR+PSQR
                  ELSE
                     SPK=SPK+(PCAP*WGT(I))
                     SPSM=SPSM+(PSMALL*WGT(I))
                     SPSQR=SPSQR+(PSQR*WGT(I))
                  ENDIF
                  IF (PQ.LE.ZERO) THEN
*                    (Fis(ik) undefined)
                     FISIK(I)=UNDEF
                     IF (ABS(PSMALL-1.).LE.ZERO) THEN
*                       (Locus monomorphic in i(th) population;
*                          Fis(i) undefined)
                        FISNUM(I)=UNDEF
                        FISDEN(I)=1.
                     ELSE
                        GO TO 505
                     ENDIF
                  ELSE
                     QFISIK=(PCAP-PSQR)/PQ
                     FISIK(I)=QFISIK
                     FISNUM(I)=FISNUM(I)+PQ*QFISIK
                     FISDEN(I)=FISDEN(I)+PQ
                  ENDIF
505            CONTINUE
               BIGP(K)=BIGP(K)+SPK
*              SMALLP(K) = Weighted allele frequency
               SMALLP(K)=SMALLP(K)+SPSM
               SMPSQ(K)=SMPSQ(K)+SPSQR
               IF (OUT1) THEN
                  CALL CONVRT (FISIK,FISROW,I1,I2,MT)
                  IF (NLET.NE.0) THEN
                     WRITE (6,209) LETTER(K),(FISROW(I),I=1,NCOLS)
                     GO TO 504
                  ENDIF
                  WRITE (6,210) K,(FISROW(I),I=1,NCOLS)
               ENDIF
            ENDIF
504      CONTINUE
         IF (OUT1) THEN
            WRITE (6,206) ('--------',I=I1,I2)
            DO 506 I=I1,I2
               FISI(I)=FISNUM(I)/FISDEN(I)
506         CONTINUE
            CALL CONVRT (FISI,FISROW,I1,I2,MT)
            WRITE (6,211) (FISROW(I),I=1,NCOLS)
            WRITE (6,206) ('--------',I=I1,I2)
            WRITE (6,212)
            IF (I2.NE.NOTU) THEN
               I1=I1+NCOL
               GO TO 9
            ENDIF
         ENDIF

*        F(is), F(it), and F(st) for individual alleles:
         IF (WEIR) GO TO 550
         IF (OUT2) WRITE (6,213) LOCNAM(J)
         FISJ=0.
         FITJ=0.
         FSTJ=0.
         DENOM1=0.
         DENOM2=0.
         DO 507 K=1,NALL
            IF (PRES(K)) THEN
               IF(UNWT) THEN
                 PCAPK=BIGP(K)/NOTU
                 PSMK=SMALLP(K)/NOTU
                 PSMKSQ=PSMK*PSMK
                 PSQK=SMPSQ(K)/NOTU
               ELSE
                 PCAPK=BIGP(K)
                 PSMK=SMALLP(K)
                 PSMKSQ=PSMK*PSMK
                 PSQK=SMPSQ(K)
               ENDIF
                 WT1=PSMK-PSQK
                 WT2=PSMK-PSMKSQ
               IF (WT1.LE.ZERO) THEN
*                 ( Fis(k) is undefined if frequency of allele k is )
*                 ( 0 or 1 in all subpopulations                    )
                  FISK=UNDEF
               ELSE
                  FISK=(PCAPK-PSQK)/WT1
                  FISJ=FISJ+WT1*FISK
               ENDIF
               FITK=(PCAPK-PSMKSQ)/WT2
               FSTK=(PSQK-PSMKSQ)/WT2
               FITJ=FITJ+WT2*FITK
               FSTJ=FSTJ+WT2*FSTK
               DENOM1=DENOM1+WT1
               DENOM2=DENOM2+WT2
               IF (OUT2) THEN
                  IF (FISK .EQ. UNDEF) THEN
                     IF (NLET.NE.0) THEN
                           WRITE (6,214) LETTER(K),FSTK,FITK
                           GO TO 507
                     ENDIF
                     WRITE (6,215) K,FSTK,FITK
                     GO TO 507
                  ENDIF
                  IF (NLET.NE.0) THEN
                      WRITE (6,216) LETTER(K),FISK,FSTK,FITK
                  ELSE
                      WRITE (6,217) K,FISK,FSTK,FITK
                  ENDIF
               ENDIF
            ENDIF
507      CONTINUE
         FISTOT=FISTOT+FISJ
         FITTOT=FITTOT+FITJ
         FSTTOT=FSTTOT+FSTJ
         DN1TOT=DN1TOT+DENOM1
         DN2TOT=DN2TOT+DENOM2
         IF (DENOM1.LE.ZERO) THEN
            FISJ=UNDEF
         ELSE
            FISJ=FISJ/DENOM1
         ENDIF
         FITJ=FITJ/DENOM2
         FSTJ=FSTJ/DENOM2
         FIS(J)=FISJ
         FIT(J)=FITJ
         FST(J)=FSTJ
         IF (OUT2) THEN
            IF (FISJ .EQ. UNDEF) THEN
               WRITE (6,218) FSTJ,FITJ
            ELSE
               WRITE (6,219) FISJ,FSTJ,FITJ
            ENDIF
         ENDIF
      IF(.NOT.WEIR) GO TO 500

C     Calculations for Weir and Cockerham's method.

 550  SUMALV=0.0                                                       
      IF (OUT2) WRITE (6,213) LOCNAM(J)
      DO 1930 IWB=1,NALL                                              
      IF(SMALLP(IWB).EQ.0.0) GO TO 1930                                
      ALLVAR=0.0                                                       
      DO 1940 IWC=1,NOTU                                               

C     EQ. 17B.                                                         
      ALLVAR=(((ABS(FREQ2(IWB,IWC)-SMALLP(IWB)))**2.0)*N2(IWC))
     + + ALLVAR                                                        
 1940 CONTINUE                                                         

      HAVG=HETER(IWB,J)/(XNHAT*POPCNT(J))                                                                      
      VARADJ=ALLVAR/((POPCNT(J)-1)*XNHAT)
C     EQ. 17A.                                                         
      AA=(SMALLP(IWB)*(1.0-SMALLP(IWB))-(((POPCNT(J)-1))*VARADJ)/
     +POPCNT(J))
      AA=AA-(HAVG/4.)
      AA=XNHAT*(VARADJ-AA/(XNHAT-1))/XNC 
C                                                                      
C     EQS. 10A AND 10B.                                                
      BA=(SMALLP(IWB)*(1.0-SMALLP(IWB))-(((POPCNT(J)-1))*VARADJ)/
     +POPCNT(J))
      BA=BA-(((2.*XNHAT)-1)*HAVG)/(4.*XNHAT)
      BA=XNHAT*BA/(XNHAT-1)  
      CA=0.5*HAVG                         
      AL(J)=AL(J)+AA
      BL(J)=BL(J)+BA
      CL(J)=CL(J)+CA
      SUMALV=ALLVAR+SUMALV                                             
C                                                                      
C     EQS. 6,10,17.                                                    
      CAPFA=1.0-(CA/(AA+BA+CA))                                        
      THETA=AA/(AA+BA+CA)
      TEST=CA+BA                                              
      IF (TEST.LE.0) THEN
         FA = UNDEF
         GO TO 555
      ENDIF
         FA=1.0-(CA/(CA+BA))
  555    IF (OUT2) THEN
            IF (FA .EQ. UNDEF) THEN
               IF (NLET.NE.0) THEN
                  WRITE(6,214) LETTER(IWB),THETA,CAPFA
                     GO TO 1930
               ENDIF
                  WRITE(6,215)IWB,FA,THETA,CAPFA
                     GO TO 1930
            ENDIF
               IF (NLET.NE.0) THEN
                  WRITE (6,216) LETTER(IWB),FA,THETA,CAPFA
               ELSE
                  WRITE(6,217)IWB,FA,THETA,CAPFA
               ENDIF
         ENDIF
 1930 CONTINUE                                                         
C                                                                      
C     EQS. 6,12, AND 19.                                               
      CAPFL(J)=1.0-(CL(J)/(AL(J)+BL(J)+CL(J)))               
      THETL(J)=AL(J)/(AL(J)+BL(J)+CL(J))
      TEST=BL(J)+CL(J)
      IF (TEST.EQ.0) THEN 
         FL(J)=UNDEF
         GO TO 540
      ELSE          
         FL(J)=1.0-(CL(J)/(BL(J)+CL(J)))
      ENDIF                          
C                                                                      
C     EQS. 14A,14B, AND 21A.                                           
 540  POPA=AL(J)+POPA                                                
      POPB=BL(J)+POPB                                                
      POPC=CL(J)+POPC
        IF (OUT2) THEN
         IF(FL(J).EQ.UNDEF) THEN                                                  
            WRITE(6,218)THETL(J),CAPFL(J)                       
            CALL PAGSKP
         ELSE 
            WRITE(6,219)FL(J),THETL(J),CAPFL(J)
         ENDIF
        ENDIF
      CALL PAGSKP
                                                                      
C     EQS. 6,14, AND 21.                                               
      CAPFP=1.0-(POPC/(POPA+POPB+POPC))                                
      THETP=(POPA/(POPA+POPB+POPC))                                    
      FP=1.0-(POPC/(POPB+POPC))                                        
  
 500  CONTINUE

*  Summary table:
      IF(WEIR) GO TO 560
      CALL PAGSKP
      WRITE (6,220)
      DO 508 J=1,NLOC
         IF (VAR(J)) THEN
            IF (FIS(J) .EQ. UNDEF) THEN
               WRITE (6,221) LOCNAM(J),FST(J),FIT(J)
            ELSE
               WRITE (6,222) LOCNAM(J),FIS(J),FST(J),FIT(J)
            ENDIF
         ENDIF
508   CONTINUE
      FISBAR=FISTOT/DN1TOT
      FITBAR=FITTOT/DN2TOT
      FSTBAR=FSTTOT/DN2TOT
      WRITE (6,223) FISBAR,FSTBAR,FITBAR
      IF(.NOT.WEIR) RETURN

C     JACKKNIFE PROCEDURE TO CALCULATE THE VARIANCE IN F-STATISTICS    
C     AMONG LOCI.                                                      
560   A=0.
      B=0.
      C=0.
      SUMCAP=0.
      SUMTHT=0.
      SUMFJK=0.
      SSQCP=0.
      SSQTH=0.
      SSQFK=0.

      DO 2060 IJACK=1,NLOC                                             
      IF(POPCNT(IJACK).GT.1)GO TO 2070
      NLOC=NLOC-1                                                      
      GO TO 2060                                                       
 2070 A=POPA-AL(IJACK)                                                 
      B=POPB-BL(IJACK)                                                 
      C=POPC-CL(IJACK)                                                 
      CFJK(IJACK)=1.0-(C/(A+B+C))                                      
      TJK(IJACK)=A/(A+B+C)                                             
      XFJK(IJACK)=1.0-(C/(B+C))                                        
      SUMCAP=SUMCAP+CFJK(IJACK)                                        
      SUMTHT=SUMTHT+TJK(IJACK)                                         
      SUMFJK=SUMFJK+XFJK(IJACK)                                        
      SSQCP=SSQCP+(ABS(CFJK(IJACK))**2.0)                              
      SSQTH=SSQTH+(ABS(TJK(IJACK))**2.0)                               
      SSQFK=SSQFK+(ABS(XFJK(IJACK))**2.0)                              
 2060 CONTINUE                                                         
      BIAS=((NLOC-1.0)/NLOC)
      ACAP=NLOC*CAPFP-(NLOC-1)*SUMCAP/NLOC
      ATHT=NLOC*THETP-(NLOC-1)*SUMTHT/NLOC
      AFIS=NLOC*FP-(NLOC-1)*SUMFJK/NLOC                                           
      VARCAP=BIAS*(SSQCP-((SUMCAP**2.)/NLOC))             
      VARTHT=BIAS*(SSQTH-((SUMTHT**2.)/NLOC))             
      VARFJK=BIAS*(SSQFK-((SUMFJK**2.)/NLOC))             
      SDCAP=SQRT(VARCAP)                                               
      SDTHT=SQRT(VARTHT)                                               
      SDFJK=SQRT(VARFJK)                                               
      CALL PAGSKP
C                                                                      
C     PRINT TABLE OF F-STATISTICS (WEIR & COCKERHAM)                   
      WRITE(6,220)                                                    
      DO 2100 IXFW=1,NLOC                                              
      IF(POPCNT(IXFW).LE.1)GO TO 2100
         IF(FL(IXFW).EQ.UNDEF) THEN
           WRITE(6,221)LOCNAM(IXFW),THETL(IXFW),CAPFL(IXFW)                                                      
         ELSE
           WRITE(6,222)LOCNAM(IXFW),FL(IXFW),THETL(IXFW),CAPFL(IXFW)                                                      
         ENDIF
 2100 CONTINUE
      WRITE(6,230)FP,THETP,CAPFP                                                         
      WRITE(6,231)AFIS,ATHT,ACAP,SDFJK,SDTHT,SDCAP                    
      RETURN

200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  F-Statistics',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')////)
201   FORMAT (' Full output requested')
202   FORMAT (' Normal output requested')
203   FORMAT (' Minimal output requested')
225   FORMAT (' Calculations made with weighted means')
226   FORMAT (' Calculations with unweighted means'///)
227   FORMAT (' Weir and Cockerham Method'///)
205   FORMAT (' FIS(IK) values'/1X,14('*')//5X,'Locus: ',A5///)
206   FORMAT (1X,'------',15A8)
207   FORMAT (9X,'------',14A8)
208   FORMAT (' Allele',I6,14I8)
209   FORMAT (3X,A1,2X,15A8)
210   FORMAT (1X,I3,2X,15A8)
211   FORMAT (' Mean',1X,15A8)
212   FORMAT (////)
213   FORMAT (' F-statistics for individual alleles'/1X,35('*')//5X,
     +'LOCUS: ',A5////1X,37('-')/' Allele',5X,'F(IS)',5X,'F(ST)',
     +5X,'F(IT)'/1X,37('-'))
214   FORMAT (3X,A1,9X,'... ',2F10.4)
215   FORMAT (1X,I3,9X,'... ',2F10.4)
216   FORMAT (3X,A1,3X,3F10.4)
217   FORMAT (1X,I3,3X,3F10.4)
218   FORMAT (1X,37('-')/' Mean',8X,'... ',2F10.4/1X,37('-')////)
219   FORMAT (1X,37('-')/' Mean',2X,3F10.4/1X,37('-')////)
220   FORMAT (' Summary of F-statistics at all loci'/1X,35('*')////1X,
     +35('-')/' Locus',5X,'F(IS)',5X,'F(ST)',5X,'F(IT)'/1X,35('-')/)
221   FORMAT (1X,A5,6X,'... ',2F10.4)
222   FORMAT (1X,A5,3F10.4)
223   FORMAT (1X,35('-')/' Mean',1X,3F10.4/1X,35('-')////)
230   FORMAT (1X,35('-')/' Mean',1X,3F10.4,/)
231   FORMAT (1X,'JACKKNIFE ESTIMATES OVER LOCI'/,
     +1X,35('-')/' Mean',1X,3F10.4/' S.D.(X)',3F10.5/
     +1X,35('-')////)
300   FORMAT (/' ERROR: Step FSTAT cannot be performed unless genotype',
     +' frequencies are available'/8X,'(DATYP=1 OR 2).  Use step',
     +' WRIGHT78 instead.')
301   FORMAT (' ERROR: Illegal value for output:',I5)
      END


      SUBROUTINE CONVRT (FIS,FISROW,I1,I2,MT)

*  Converts a row of F(is) values to a character string to allow
*  clearer representation of undefined values

      REAL FIS
      CHARACTER FISROW*8
      DIMENSION FIS(MT),FISROW(15)

      ICOL=0
      DO 500 I=I1,I2
         ICOL=ICOL+1
         IF (FIS(I).EQ.99.E9) THEN
            FISROW(ICOL)='     ...'
         ELSE
            WRITE (FISROW(ICOL),'(F8.4)') FIS(I)
         ENDIF
500   CONTINUE
      RETURN
      END
